#include "../../kernel/system.h"

/* Handles the keyboard interrupt */

void keyboard_handler(struct regs *r)
{
    unsigned char kbdus[128] =
{
    0,  27, '1', '2', '3', '4', '5', '6', '7', '8',	/* 9 */
  '9', '0', '-', '=', '\b',	/* Backspace */
  '\t',			/* Tab */
  'q', 'w', 'e', 'r',	/* 19 */
  't', 'y', 'u', 'i', 'o', 'p', '[', ']', '\n',		/* Enter key */
    0,			/* 29   - Control */
  'a', 's', 'd', 'f', 'g', 'h', 'j', 'k', 'l', ';',	/* 39 */
 '\'', '`',   0,		/* Left shift */
 '\\', 'z', 'x', 'c', 'v', 'b', 'n',			/* 49 */
  'm', ',', '.', '/',   0,					/* Right shift */
  '*',
    0,	/* Alt */
  ' ',	/* Space bar */
    0,	/* Caps lock */
    0,	/* 59 - F1 key ... > */
    0,   0,   0,   0,   0,   0,   0,   0,
    0,	/* < ... F10 */
    0,	/* 69 - Num lock*/
    0,	/* Scroll Lock */
    0,	/* Home key */
    0,	/* Up Arrow */
    0,	/* Page Up */
  '-',
    0,	/* Left Arrow */
    0,
    0,	/* Right Arrow */
  '+',
    0,	/* 79 - End key*/
    0,	/* Down Arrow */
    0,	/* Page Down */
    0,	/* Insert Key */
    0,	/* Delete Key */
    0,   0,   0,
    0,	/* F11 Key */
    0,	/* F12 Key */
    0,	/* All other keys are undefined */
};
    
unsigned char kbdusS[128] =
{
    0,  27, '!', '@', '#', '$', '%', '^', '&', '*',	/* 9 */
  '(', ')', '_', '+', '\b',	/* Backspace */
  '\t',			/* Tab */
  'Q', 'W', 'E', 'R',	/* 19 */
  'T', 'Y', 'U', 'I', 'O', 'P', '{', '}', '\n',		/* Enter key */
    0,			/* 29   - Control */
  'A', 'S', 'D', 'F', 'G', 'H', 'J', 'K', 'L', ':',	/* 39 */
 '"', '~',   0,		/* Left shift */
 '|', 'Z', 'X', 'C', 'V', 'B', 'N',			/* 49 */
  'M', '<', '>', '?',   0,					/* Right shift */
  '*',
    0,	/* Alt */
  ' ',	/* Space bar */
    0,	/* Caps lock */
    0,	/* 59 - F1 key ... > */
    0,   0,   0,   0,   0,   0,   0,   0,
    0,	/* < ... F10 */
    0,	/* 69 - Num lock*/
    0,	/* Scroll Lock */
    0,	/* Home key */
    0,	/* Up Arrow */
    0,	/* Page Up */
  '-',
    0,	/* Left Arrow */
    0,
    0,	/* Right Arrow */
  '+',
    0,	/* 79 - End key*/
    0,	/* Down Arrow */
    0,	/* Page Down */
    0,	/* Insert Key */
    0,	/* Delete Key */
    0,   0,   0,
    0,	/* F11 Key */
    0,	/* F12 Key */
    0,	/* All other keys are undefined */
};
    
    unsigned char scancode;

    /* Read from the keyboard's data buffer */
    scancode = inb(0x60);

    if (scancode & 0x80)
    {
        //Key released
        switch (scancode) {
            case 0xaa :
                shiftKey = 0;
                break;
            case 0xb6 :
                shiftKey = 0;
                break;
            case 0xb8 :
//                monPutchar('Alt');
                break;
            case 0x9d :
//                monPutchar('Ctrl');
                break;
            case 0xdb :
//                monPutchar('CmdL');
                break;
            case 0xdc :
//                monPutchar('CmdR');
                break;
        }
    }
    else
    {
        //A key was just pressed.
        switch (scancode) {
            case 0x2a :
                shiftKey = 1;
                break;
            case 0x36 :
                shiftKey = 1;
                break;
            case 0x3a :
                if (capsLock == 0) capsLock = 1;
                else if (capsLock == 1) capsLock = 0;
                break;
            case 0x48 :
                if (cursorY == 0) cursorY = 24;
                else cursorY--;
                break;
            case 0x50 :
                if (cursorY == 24) cursorY = 0;
                else cursorY++;
                break;
            case 0x4b :
                if (cursorX == 0) cursorX = 79;
                else cursorX--;
                break;
            case 0x4d :
                if (cursorX == 79) cursorX = 0;
                else cursorX++;
                break;
            case 0x38 :
//                monPutchar('Alt');
                break;
            case 0x1d :
//                monPutchar('Ctrl');
                break;
            case 0x5b :
//                monPutchar('CmdL');
                break;
            case 0x5c :
//                monPutchar('CmdR');
                break;
        }
        if (shiftKey == 1) monPut(kbdusS[scancode]);
        else monPut(kbdus[scancode]);
    }
}

/* Installs the keyboard handler into IRQ1 */
void keyboard_install()
{
    irq_install_handler(1, keyboard_handler);
}
